﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.IO;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Backend.Csv {
  [StorableClass]
  public class CsvImportCommand : DataSetCommandBase {
    private ColumnGroup newColumnGroup;

    [StorableConstructor]
    protected CsvImportCommand(bool deserializing) : base(deserializing) { }

    public CsvImportCommand(DataSet dataSet)
      : base(dataSet) {
    }

    [Storable]
    private string fileName;
    public string FileName {
      get { return fileName; }
      set { this.fileName = value; }
    }

    [Storable]
    private char[] delimiter;
    public char[] Delimiter {
      get { return this.delimiter; }
      set { this.delimiter = value; }
    }

    [Storable]
    private bool columnNamesInFirstRow;
    public bool ColumnNamesInFirstRow {
      get { return this.columnNamesInFirstRow; }
      set { this.columnNamesInFirstRow = value; }
    }

    public override string Description {
      get { return "Import CSV file"; }
    }

    public override void Execute() {
      base.Execute();
      newColumnGroup = new ColumnGroup(fileName.Substring(fileName.LastIndexOf('\\') + 1));
      StreamReader reader = new StreamReader(fileName);
      string firstLine = reader.ReadLine();

      int i = 1;
      string[] values = firstLine.Split(delimiter);
      foreach (string value in values) {
        if (columnNamesInFirstRow)
          newColumnGroup.AddColumn(new StringColumn(value));
        else {
          newColumnGroup.AddColumn(new StringColumn("Column " + i));
        }
        i++;
      }
      if (!columnNamesInFirstRow)
        newColumnGroup.AddRow(values);
      while (!reader.EndOfStream) {
        newColumnGroup.AddRow(reader.ReadLine().Split(delimiter));
      }
      reader.Close();
      DataSet.AddColumnGroup(newColumnGroup);
      DataSet.FireChanged();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      DataSet.RemoveColumnGroup(newColumnGroup);
      newColumnGroup = null;
      DataSet.FireChanged();
    }
  }
}
